-- User Interface

local function L(x) return Translations.BankIt.L(x) end

local miniWindow
local progressWindow
local configWindow
local context

function BankIt.BuildProgressWindow()
	progressWindow = UI.CreateFrame("Text", "progress", context)
	progressWindow:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	progressWindow:SetWidth(500)
	progressWindow:SetHeight(50)
	progressWindow:SetText("...")
	progressWindow:SetFontSize(30)
	progressWindow:SetBackgroundColor(1, 0, 0, 0.5)
	progressWindow:SetVisible(false)
end

function BankIt.ShowProgressWindow()
	progressWindow:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	progressWindow:SetVisible(true)
	miniWindow.stop:SetVisible(true)
end

function BankIt.HideProgressWindow()
	progressWindow:SetVisible(false)
	miniWindow.stop:SetVisible(false)
end

local configselectedtype = 1		-- invbag, bankbag, guild, vault
local configmaxbags=7
local configselectedbag = 1
local configmaxslots=36
local radiobuttons = {}

local function makebutton(parent, x, y, width, group, label, func)

	local button=UI.CreateFrame("Frame", "F"..label, parent)
	button:SetPoint("TOPLEFT", parent, "TOPLEFT", x, y)
	button:SetWidth(width); button:SetHeight(25)
	button:SetBackgroundColor(77/255, 136/255, 168/255)

	local text=UI.CreateFrame("Text", "T"..label, button)
	text:SetPoint("CENTER", button, "CENTER", 0, 0)
	text:SetFontSize(14); text:SetText(label)
	text:SetFontColor(1, 1, 1)

	button:EventAttach(Event.UI.Input.Mouse.Cursor.In, function()
		button:SetBackgroundColor(138/255, 202/255, 195/255)
	end, "mousein", 0)
	button:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function()
		button:SetBackgroundColor(77/255, 136/255, 168/255)
	end, "mousein", 0)
	
	if not radiobuttons[group] then
		radiobuttons[group]={}
		text:SetFontColor(1, 0, 0)
	end
	table.insert(radiobuttons[group], text)
	button:EventAttach(Event.UI.Input.Mouse.Left.Down, function()
		if func() then
			for i=1, #radiobuttons[group], 1 do
				radiobuttons[group][i]:SetFontColor(1, 1, 1)
			end
			text:SetFontColor(1, 0, 0)
		end
	end, "mousepress", 0)
end

local function makestatic(parent, x, y, label, width)
	local text=UI.CreateFrame("Text", "S"..label, parent)
	text:SetPoint("TOPLEFT", parent, "TOPLEFT", x, y)
	text:SetFontSize(14); text:SetText(label)
	text:SetFontColor(0.7, 0.7, 0.7)
	if (width) then text:SetWidth(width) end
	return text
end

local function tempitemslotvault(a, b)
	if Utility.Item.Slot.Vault then return Utility.Item.Slot.Vault(a, b) end
	return Utility.Item.Slot.Bank("main", b)
end

local function redrawslots()
	local position=math.floor(configWindow.sbar:GetPosition())
	local func
	local i
	if     configselectedtype==1 then func=Utility.Item.Slot.Inventory
	elseif configselectedtype==2 then func=Utility.Item.Slot.Vault
	elseif configselectedtype==3 then func=Utility.Item.Slot.Bank
	else                              func=Utility.Item.Slot.Guild
	end
	for i=1, 20, 1 do
		local slot=func(configselectedbag, position+i)
		local item=Inspect.Item.Detail(slot)
		local designation=""
		local itemname=""
		local itemcat=""
		if item and item.name then itemname=item.name end
		if item and item.category then itemcat=item.category end
		if BankItChar.designations[slot] then designation=BankItChar.designations[slot] end

		configWindow.rows[i].slotno:SetText(tostring(position+i))
		configWindow.rows[i].designation:SetText(designation)
		configWindow.rows[i].sitem:SetText(tostring(itemname))
		configWindow.rows[i].stype:SetText(tostring(itemcat))
		configWindow.rows[i].slotname=slot
	end
	
	local ignored={}
	local n=1
	for k, v in pairs(BankItChar.ignored) do ignored[n]=k; n=n+1 end
	table.sort(ignored)
	for i=1, 20, 1 do
		if ignored[i] then
			configWindow.rows[i].ignore:SetText(ignored[i])
		else
			configWindow.rows[i].ignore:SetText("")
		end
	end
end

local function rebuildslots()
	local scrollableslots=configmaxslots-20
	configWindow.sbar:SetRange(0, scrollableslots)
	configWindow.sbar:SetThickness(20)
	if configWindow.sbar:GetPosition()==0 then
		redrawslots()
	else
		configWindow.sbar:SetPosition(0)
	end
end

local function makeaction(parent, x, y, label, func)
	local text=UI.CreateFrame("Text", "A"..label, parent)
	text:SetPoint("TOPLEFT", parent, "TOPLEFT", x, y)
	text:SetFontSize(14); text:SetText(label)
	text:SetFontColor(0.7, 0.7, 0.7)
	text:SetBackgroundColor(77/255, 136/255, 168/255)

	text:EventAttach(Event.UI.Input.Mouse.Cursor.In, function()
		text:SetBackgroundColor(138/255, 202/255, 195/255)
	end, "mousein", 0)
	text:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function()
		text:SetBackgroundColor(77/255, 136/255, 168/255)
	end, "mousein", 0)	
	if func then text:EventAttach(Event.UI.Input.Mouse.Left.Down, function()
		func()
		redrawslots()
	end, "mousepress", 0) end
	return text
end

local function makeMovableAndClosable(window)

	local closeButton = UI.CreateFrame("RiftButton", "CloseButton", window)
	closeButton:SetSkin("close")
	closeButton:SetPoint("TOPRIGHT", window, "TOPRIGHT", -5, 15)
	function closeButton:EventLeftClick(handle)
		window:Close()
	end
	closeButton:EventAttach(Event.UI.Input.Mouse.Left.Click, closeButton.EventLeftClick, "LeftClick", -2)

	local content=window:GetContent()
	window.attachx=0
	window.attachy=0
	content:EventAttach(Event.UI.Input.Mouse.Left.Down, function ()
		-- print("down");
		local mouse=Inspect.Mouse()
		window.origmousex=mouse.x
		window.origmousey=mouse.y
		window.moving=true
	end, "LeftDown", -2)
	content:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function()
		-- print("move");
		if not window.moving then return end
		local mouse=Inspect.Mouse()
		local deltax=mouse.x-window.origmousex
		local deltay=mouse.y-window.origmousey
		window:SetPoint("CENTER", UIParent, "CENTER", window.attachx+deltax, window.attachy+deltay)
	end, "MouseMove", -2)
	content:EventAttach(Event.UI.Input.Mouse.Left.Up, function ()
		-- print("up");
		if not window.moving then return end
		window.moving=false
		local mouse=Inspect.Mouse()
		local deltax=mouse.x-window.origmousex
		local deltay=mouse.y-window.origmousey
		window.attachx=window.attachx+deltax
		window.attachy=window.attachy+deltay
		window:SetPoint("CENTER", UIParent, "CENTER", window.attachx, window.attachy)
	end, "LeftUp", -2)
	content:EventAttach(Event.UI.Input.Key.Down.Dive, function(handle, key)
		print("c"); dump(key)
	end, "KeyDown", 999)
	window:EventAttach(Event.UI.Input.Key.Down.Dive, function(handle, key)
		print("w"); dump(key)
	end, "KeyDown", 999)
	-- print("MakeMovableAndClosable finished, masking=" .. content:GetMouseMasking())
end

function BankIt.BuildConfigWindow()
	configWindow=UI.CreateFrame("RiftWindow", "BankitConfiguration", context)
	configWindow:SetTitle(L("BankIt Configuration"));
	configWindow:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	configWindow:SetWidth(700)
	configWindow:SetVisible(false)
	makeMovableAndClosable(configWindow)
	function configWindow.Close() configWindow:SetVisible(false) end
	
	makebutton(configWindow,  20, 50, 90, "type", L("Inventory"), function() configselectedtype=1 configmaxbags=7 configselectedbag=1 configmaxslots=36  rebuildslots() return true end)
	makebutton(configWindow, 120, 50, 90, "type", L("Vault"    ), function() configselectedtype=2 configmaxbags=6 configselectedbag=1 configmaxslots=32  rebuildslots() return true end)
	makebutton(configWindow, 220, 50, 90, "type", L("Bank bag" ), function() configselectedtype=3 configmaxbags=8 configselectedbag=1 configmaxslots=36  rebuildslots() return true end)
	makebutton(configWindow, 320, 50, 90, "type", L("Guild"    ), function() configselectedtype=4 configmaxbags=6 configselectedbag=1 configmaxslots=105 rebuildslots() return true end)
	
	makebutton(configWindow,  20, 80, 40, "bag", "1", function() if configmaxbags>=1 then configselectedbag=1 rebuildslots() return true else return false end end)
	makebutton(configWindow,  70, 80, 40, "bag", "2", function() if configmaxbags>=2 then configselectedbag=2 rebuildslots() return true else return false end end)
	makebutton(configWindow, 120, 80, 40, "bag", "3", function() if configmaxbags>=3 then configselectedbag=3 rebuildslots() return true else return false end end)
	makebutton(configWindow, 170, 80, 40, "bag", "4", function() if configmaxbags>=4 then configselectedbag=4 rebuildslots() return true else return false end end)
	makebutton(configWindow, 220, 80, 40, "bag", "5", function() if configmaxbags>=5 then configselectedbag=5 rebuildslots() return true else return false end end)
	makebutton(configWindow, 270, 80, 40, "bag", "6", function() if configmaxbags>=6 then configselectedbag=6 rebuildslots() return true else return false end end)
	makebutton(configWindow, 320, 80, 40, "bag", "7", function() if configmaxbags>=7 then configselectedbag=7 rebuildslots() return true else return false end end)
	makebutton(configWindow, 370, 80, 40, "bag", "8", function() if configmaxbags>=8 then configselectedbag=8 rebuildslots() return true else return false end end)

	-- makestatic(configWindow, 250, 110, L("current"))
	makestatic(configWindow,  20, 130, L("Slot"))
	makestatic(configWindow,  60, 130, L("Designation"))
	makestatic(configWindow, 200, 130, L("Item"))
	makestatic(configWindow, 330, 130, L("Category"))
	makestatic(configWindow, 500, 130, L("Ignore list"))
	
	configWindow.rows={}
	local scroller=configWindow		--
	for i=1, 20, 1 do
		local y=150+i*17
		configWindow.rows[i]={}
		configWindow.rows[i].slotno	=makestatic(scroller,  20, y, tostring(i) )
		configWindow.rows[i].designation=makestatic(scroller,  55, y, "desig"     , 90)
		configWindow.rows[i].cleardesig	=makeaction(scroller, 150, y, "X"         , function() BankItChar.designations[configWindow.rows[i].slotname]=nil; end)
		configWindow.rows[i].ignore	=makeaction(scroller, 165, y, "I"         , function() BankItChar.designations[configWindow.rows[i].slotname]="ignore"; end)
		
		configWindow.rows[i].ditem      =makeaction(scroller, 180, y, "<"         , function() BankItChar.designations[configWindow.rows[i].slotname]=configWindow.rows[i].sitem:GetText(); end)
		configWindow.rows[i].sitem      =makestatic(scroller, 195, y, "itemname"  , 90)
		configWindow.rows[i].iitem      =makeaction(scroller, 290, y, ">"         , function() BankItChar.ignored[configWindow.rows[i].sitem:GetText()]=true; end)

		configWindow.rows[i].dtype      =makeaction(scroller, 310, y, "<"         , function() BankItChar.designations[configWindow.rows[i].slotname]=configWindow.rows[i].stype:GetText(); end)
		configWindow.rows[i].stype      =makestatic(scroller, 325, y, "itemtype"  , 90)
		configWindow.rows[i].itype      =makeaction(scroller, 420, y, ">"         , function() BankItChar.ignored[configWindow.rows[i].stype:GetText()]=true; end)
		
		configWindow.rows[i].ignore	=makestatic(scroller, 500, y, "ignore"	  ,160)
		configWindow.rows[i].iignore	=makeaction(scroller, 670, y, "X"	  , function() BankItChar.ignored[configWindow.rows[i].ignore:GetText()]=nil; end)
		
		configWindow.rows[i].slotname   =""
	end
	
	local sbar=UI.CreateFrame("RiftScrollbar", "scrollslots", configWindow)
	sbar:SetOrientation("vertical")
	sbar:SetPoint("TOPLEFT", configWindow, "TOPLEFT", 450, 165)
	sbar:SetPoint("BOTTOMRIGHT", configWindow, "TOPLEFT", 465, 500)
	sbar:EventAttach(Event.UI.Scrollbar.Change, redrawslots, "redrawslots", 0)
	configWindow.sbar=sbar
	
	
	rebuildslots()
	redrawslots()
end

function BankIt.ShowConfigWindow()
	configWindow:SetVisible(true)
end

function BankIt.HideConfigWindow()
	configWindow:SetVisible(false)
end

function BankIt.SetProgressWindowText(text)
	progressWindow:SetText(text)
end

function BankIt.BuildMiniWindow()
	miniWindow=UI.CreateFrame("Frame", "BankIt", context)
	miniWindow:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 0, 0)
	miniWindow:SetWidth(120)
	miniWindow:SetHeight(30)
	miniWindow:SetBackgroundColor(1.0, 0.0, 0.0, 1)
	miniWindow:SetVisible(false)

	miniWindow.push = UI.CreateFrame("Texture", "push", miniWindow)
	miniWindow.push:SetPoint("TOPLEFT", miniWindow, "TOPLEFT",   0, 0)
	miniWindow.push:SetWidth(30)
	miniWindow.push:SetHeight(30)
	miniWindow.push:SetTexture("BankIt", "push.png")
	miniWindow.push:SetVisible(true)
	
	miniWindow.put = UI.CreateFrame("Texture", "put", miniWindow)
	miniWindow.put:SetPoint("TOPLEFT", miniWindow, "TOPLEFT",  30, 0)
	miniWindow.put:SetWidth(30)
	miniWindow.put:SetHeight(30)
	miniWindow.put:SetTexture("BankIt", "put.png")
	miniWindow.put:SetVisible(true)
	
	miniWindow.get = UI.CreateFrame("Texture", "get", miniWindow)
	miniWindow.get:SetPoint("TOPLEFT", miniWindow, "TOPLEFT",  60, 0)
	miniWindow.get:SetWidth(30)
	miniWindow.get:SetHeight(30)
	miniWindow.get:SetTexture("BankIt", "get.png")
	miniWindow.get:SetVisible(true)
	
	miniWindow.pull = UI.CreateFrame("Texture", "pull", miniWindow)
	miniWindow.pull:SetPoint("TOPLEFT", miniWindow, "TOPLEFT", 90, 0)
	miniWindow.pull:SetWidth(30)
	miniWindow.pull:SetHeight(30)
	miniWindow.pull:SetTexture("BankIt", "pull.png")
	miniWindow.pull:SetVisible(true)

	miniWindow.wrench = UI.CreateFrame("Texture", "wrench", miniWindow)
	miniWindow.wrench:SetPoint("TOPLEFT", miniWindow, "TOPLEFT", 130, 0)
	miniWindow.wrench:SetWidth(30)
	miniWindow.wrench:SetHeight(30)
	miniWindow.wrench:SetTexture("BankIt", "wrench.png")
	miniWindow.wrench:SetVisible(true)

	miniWindow.stop = UI.CreateFrame("Texture", "stop", miniWindow)
	miniWindow.stop:SetPoint("TOPLEFT", miniWindow, "TOPLEFT", 190, 0)
	miniWindow.stop:SetWidth(30)
	miniWindow.stop:SetHeight(30)
	miniWindow.stop:SetTexture("BankIt", "stop.png")
	miniWindow.stop:SetVisible(false)

	function miniWindow.push.Event:LeftUp()
		BankIt.swap(1)
	end
	function miniWindow.put.Event:LeftUp()
		BankIt.swap(2)
	end
	function miniWindow.get.Event:LeftUp()
		BankIt.swap(3)
	end
	function miniWindow.pull.Event:LeftUp()
		BankIt.swap(4)
	end
	function miniWindow.wrench.Event:LeftUp()
		if configWindow:GetVisible() then
			configWindow:SetVisible(false)
		else
			configWindow:SetVisible(true)
			rebuildslots()
		end
	end
	function miniWindow.stop.Event:LeftUp()
		BankIt.stop()
	end
end

function BankIt.ShowMiniWindow(frame)
	miniWindow:SetPoint("TOPLEFT", frame, "TOPLEFT", 20, -30)
	miniWindow:SetVisible(true)
end

function BankIt.HideMiniWindow()
	miniWindow:SetVisible(false)
end

function BankIt.createUI()
	if context == nil then
		context=UI.CreateContext("BankIt")
	end

	if (miniWindow == nil) then
		BankIt.BuildMiniWindow()
	end
	if (progressWindow == nil) then
		BankIt.BuildProgressWindow()
	end
	if (configWindow == nil) then
		BankIt.BuildConfigWindow()
	end
end
