local AddonData, PrivateTable = ...

if BankIt then
	print ("BankIt already loaded")
	return
end

BankIt = {
	version = AddonData.toc.Version,
	debugMode = false,
}

local function L(x) return Translations.BankIt.L(x) end

local function ensureVariablesInited()
	if not BankItChar then BankItChar={} end
	if not BankItChar.ignored then BankItChar.ignored={} end
	if not BankItChar.designations then BankItChar.designations={} end	
	if not BankItShard then BankItShard={} end
	if not BankItGlobal then BankItGlobal={} end
	if not BankItGlobal.perFrame then BankItGlobal.perFrame=10 end
end

local function interactionChanged(type, state)
	if type == "bank" and state == true then
		BankIt.ShowMiniWindow(UI.Native.Bank)
	elseif type=="guildbank" and state == true then
		BankIt.ShowMiniWindow(UI.Native.BankGuild)
	else
		BankIt.HideMiniWindow()
	end
end
		
function BankIt.printVersion()
	print(L("BankIt Version ") .. (BankIt.version) .. L(" installed!"))
end

local bankstrs={"main", 1, 2, 3, 4, 5, 6, 7, 8}
local bagstrs ={1, 2, 3, 4, 5}

local bankstodo= {}
local bagstodo = {}
local desigtodo = {}
local modetodo = nil
local banktype = ""

local waitingForUpdate = {}
local function moveComplete(updates)
	if BankIt.debugMode then
		print("Item Slot Event")
		dump(waitingForUpdate)
		dump(updates)
	end
	for k, v in pairs(updates) do
		waitingForUpdate[k]=nil
	end
	if BankIt.debugMode then
		print("After Event")
		dump(waitingForUpdate)
		print ("-------------------")
	end
end

local function moveComplete2(updates)
	if BankIt.debugMode then
		print("Item Update Event")
		dump(waitingForUpdate)
		dump(updates)
	end
	for k, v in pairs(updates) do
		waitingForUpdate[k]=nil
	end
	if BankIt.debugMode then
		print("After Event")
		dump(waitingForUpdate)
		print ("-------------------")
	end
end

local function systemUpdate(handle)
	if next(waitingForUpdate) ~= nil then return end
	
	local count
	for count=1, BankItGlobal.perFrame, 1 do
		if #bankstodo > 0 then
			local temp=bankstodo[1]
			local bkslot=temp[1]
			local bkitem=temp[2]
			if bkitem then
				BankIt.SetProgressWindowText(bkslot..": "..bkitem.name)
			end

			local i
			for i=1, #bagstodo, 1 do
				local temp=bagstodo[i]
				local bgslot=temp[1]
				local bgitem=temp[2]

	-- We mark completely moved items with a stack size of 0. This stack size
	-- can't be present normally, so it saves us from setting the whole item to nil.
	-- Since moving takes a while, we can't just Inspect.Item.Detail the bag/bank
	-- slots after moving; instead, we have to calculate new sizes ourselves.

				if  bgitem~= nil and bkitem ~= nil
				and bgitem.stack and bgitem.stack > 0
				and bkitem.stack and bkitem.stack > 0
				and bkitem.type == bgitem.type
				and (not bgitem.name or not BankItChar.ignored[bgitem.name])
				and (not bgitem.category or not BankItChar.ignored[bgitem.category])
				then
					if BankIt.debugMode then
						print (bkslot .. " equals " .. bgslot .. 
							":" ..  bkitem.name ..
							", bank=" ..bkitem.stack ..
							", bag=" .. bgitem.stack ..
							", max=" .. (bkitem.stackMax or "??"))
					end
					if modetodo == 1 then		-- move bag to bank, as many as possible. Personal bank will stack to 99, Guild bank will fail!
						if (banktype == "bank" and bkitem.stack < bkitem.stackMax)
						or (banktype == "guild" and bkitem.stack + bgitem.stack <= bkitem.stackMax)
						then
							print (L("Moving ")..bkitem.name..L(" from ")..bgslot..L(" to ").. bkslot)
							Command.Item.Move(bgslot, bkslot)
							bkitem.stack = bkitem.stack + bgitem.stack
							bgitem.stack = 0
							if bkitem.stack > bkitem.stackMax then
								bgitem.stack = bkitem.stack - bkitem.stackMax
								bkitem.stack = bkitem.stackMax
							end
							waitingForUpdate={[bgslot]=true}
							if banktype ~= "guild" then waitingForUpdate[bkslot]=true end
							return
						end
					elseif modetodo==2 then		-- move bag to bank if bank stack has enough space to take bag stack
						if bkitem.stack + bgitem.stack <= bkitem.stackMax then
							print (L("Moving ")..bkitem.name..L(" from ")..bgslot..L(" to ").. bkslot)
							Command.Item.Move(bgslot, bkslot)
							bkitem.stack = bkitem.stack + bgitem.stack
							bgitem.stack = 0
							waitingForUpdate={[bgslot]=true}
							if banktype ~= "guild" then waitingForUpdate[bkslot]=true end
							return
						end
					elseif modetodo==3 then		-- move bank to bag if bag stack has enough space to take bank stack
						if bkitem.stack + bgitem.stack < bgitem.stackMax then
							print (L("Moving ")..bkitem.name..L(" from ")..bkslot..L(" to ").. bgslot)
							Command.Item.Move(bkslot, bgslot)
							bgitem.stack = bkitem.stack + bgitem.stack
							bkitem.stack = 0
							waitingForUpdate={[bgslot]=true}
							if banktype ~= "guild" then waitingForUpdate[bkslot]=true end
							return
						end
					elseif modetodo == 4 then		-- move bank to bag, as many as possible
						if (banktype == "bank" and bgitem.stack < bgitem.stackMax)
						or (banktype == "guild" and bgitem.stack + bkitem.stack <= bgitem.stackMax)
						then
							print (L("Moving ")..bgitem.name..L(" from ")..bkslot..L(" to ").. bgslot)
							Command.Item.Move(bkslot, bgslot)
							bgitem.stack = bgitem.stack + bkitem.stack
							bkitem.stack = 0
							if bgitem.stack > bgitem.stackMax then
								bkitem.stack = bgitem.stack - bgitem.stackMax
								bgitem.stack = bgitem.stackMax
							end
							waitingForUpdate={[bgslot]=true}
							if banktype ~= "guild" then waitingForUpdate[bkslot]=true end
							return
						end
					end
				end
			end
			table.remove(bankstodo, 1)
		elseif #desigtodo > 0 then 
			local temp=desigtodo[1]
			local bkslot=temp[1]
			local bkitem=temp[2]
			local displaystr
			if 	bkitem and bkitem.name	then displaystr=bkitem.name
			elseif	desigtodo[bkslot] 	then displaystr=desigtodo[bkslot]
			else   				     displaystr=""
			end
			BankIt.SetProgressWindowText(bkslot..": "..displaystr)

			local i
			for i=1, #bagstodo, 1 do
				local temp=bagstodo[i]
				local bgslot=temp[1]
				local bgitem=temp[2]

				if ( modetodo == 1 or modetodo == 2)	-- move bag to bank
				and bkitem==nil 
				and bgitem~=nil
				and bgitem.stack and bgitem.stack > 0	-- might have been moved in 1st loop
				and BankItChar.designations[bkslot]
				and ( BankItChar.designations[bkslot]==bgitem.type
				   or BankItChar.designations[bkslot]==bgitem.category )
				then		
					print (L("Moving ")..bgitem.name..L(" from ")..bgslot..L(" to ").. bkslot)
					Command.Item.Move(bgslot, bkslot)
					desigtodo[1][2]=bgitem
					bagstodo[i][2]=nil
					waitingForUpdate={[bgslot]=true}
					if banktype ~= "guild" then waitingForUpdate[bkslot]=true end
					return
				elseif ( modetodo == 3 or modetodo == 4) -- move bank to bag
				and bgitem==nil
				and bkitem~=nil
				and BankItChar.designations[bgslot]
				and ( BankItChar.designations[bgslot]==bkitem.type
				   or BankItChar.designations[bgslot]==bkitem.category )
				then		
					print (L("Moving ")..bkitem.name..L(" from ")..bkslot..L(" to ").. bgslot)
					Command.Item.Move(bkslot, bgslot)
					bagstodo[i][2]=bkitem
					desigtodo[1][2]=nil
					waitingForUpdate={[bgslot]=true}
					if banktype ~= "guild" then waitingForUpdate[bkslot]=true end
					return
				end

			end
			table.remove(desigtodo, 1)
		else
			BankIt.HideProgressWindow()
			Command.Event.Detach(Event.System.Update.Begin, systemUpdate, "SystemUpdate")
			return
		end
	end
end

-- This function just fills an array of non-empty bank slots, with 
-- the frame function below processing them one by one, to avoid
-- getting performance errors when we do (9*32) bank items times (5*32)
-- inventory items in one loop.

function BankIt.swap(mode)
	local bank=Inspect.Interaction("bank")
	local guild=Inspect.Interaction("guildbank")
	
	if (bank or guild) then
		bankstodo={}
		bagstodo={}
		modetodo=mode

		local bk, bg, bki, bgi
		local bkitem, bgitem
		local bkslot, bgslot
		
		if bank then
			banktype="bank"
			for bk=1,8,1 do
				for bki = 1,32,1 do
					bkslot=Utility.Item.Slot.Bank(bk, bki)
					bkitem=Inspect.Item.Detail(bkslot)
					table.insert(bankstodo, { bkslot, bkitem })
					table.insert(desigtodo, { bkslot, bkitem })
				end
			end
			for bk=1,6,1 do
				for bki = 1,32,1 do
					bkslot=Utility.Item.Slot.Vault(bk, bki)
					bkitem=Inspect.Item.Detail(bkslot)
					table.insert(bankstodo, { bkslot, bkitem })
					table.insert(desigtodo, { bkslot, bkitem })
				end
			end
		elseif guild then
			Command.System.Watchdog.Quiet()
			banktype="guild"
			for i = 1,6,1 do
				for bki = 1,105,1 do    	        	-- 105?? 15*7?
					bkslot=Utility.Item.Slot.Guild(i, bki)
					bkitem=Inspect.Item.Detail(bkslot)
					table.insert(bankstodo, { bkslot, bkitem })
					table.insert(desigtodo, { bkslot, bkitem })
				end
			end
		end
		
		for bg = 1,7 do
		    for bgi = 1,32,1 do
		        bgslot=Utility.Item.Slot.Inventory(bg, bgi)
				bgitem=Inspect.Item.Detail(bgslot)
				table.insert(bagstodo, { bgslot, bgitem })
		    end
		end
		BankIt.ShowProgressWindow()
		Command.Event.Attach(Event.System.Update.Begin, systemUpdate, "SystemUpdate")
	end
end

function BankIt.stop()
	bagstodo = {}
	bankstodo = {}
	waitingForUpdate = {}
end

function BankIt.SlashHandler(args)
	local r = {}
	local numargs = 1
	local inquote = false
	local token, tmptoken
	for token in string.gmatch(args, "[^%s]+") do
		--print(token)
		if token:sub(1, 1) == "\"" then
			tmptoken=""
			token=token:sub(2) -- handle "abc" case
			inquote=true
			--print("start qoute token="..token)
		end
		if inquote then
			--print ("in quote, last char: "..token:sub(-1))
			if token:sub(-1) == "\"" then
				inquote=false
				token=token:sub(1, -2)
				token=tmptoken .. token
				--print ("combined string is "..token)
			else
				tmptoken=tmptoken .. token .. " "
				--print ("tmp token is "..token)
			end
		end
		if not inquote then
			r[numargs] = token
			-- print ("r["..numargs.."] ='"..token.."'")
			numargs=numargs+1
		end
	end
	local usage="/bankit version | push | pull | get | put | perframe <n> | stop | debug | ignore <item|type> | unignore <item>"
	if numargs>1 then
		if r[1] == "version" then
			BankIt.printVersion()
		elseif r[1] == "push" then
			BankIt.swap(1)
		elseif r[1] == "put" then
			BankIt.swap(2)
		elseif r[1] == "get" then
			BankIt.swap(3)
		elseif r[1] == "pull" then
			BankIt.swap(4)
		elseif r[1] == "perframe" then
			BankItGlobal.perFrame=tonumber(r[2])
		elseif r[1] == "stop" then
			BankIt.stop()
		elseif r[1] == "debug" then
			BankIt.debugMode=true
		elseif r[1] == "ignore" then
			print (tostring(r[2]))
			if r[2]==nil then
				print ("listing")
				for k, v in pairs(BankItChar.ignored) do
					print (L("ignoring ")..k)
				end
			else
				BankItChar.ignored[r[2]]=true
				print (L("will not move ")..r[2]..L(" anymore"))
			end
		elseif r[1] == "unignore" and r[2]~=nil then
			BankItChar.ignored[r[2]]=nil
			print (L("will move ")..r[2]..L(" again"))
		elseif r[1] == "designate"
		   and (r[2] == "i" or r[2] == "b" or r[2] == "g" or r[2] == "v")
		   and  tonumber(r[3])>0	-- bag / page start
		   and  tonumber(r[4])>0	-- bag / page end
		   and  tonumber(r[5])>0	-- slot start
		   and  tonumber(r[6])>0	-- slot end
		then
			local f
			if r[2]=="i" then f=Utility.Item.Slot.Inventory
			elseif r[2]=="b" then f=Utility.Item.Slot.Bank
			elseif r[2]=="g" then f=Utility.Item.Slot.Guild
			elseif r[2]=="v" then f=Utility.Item.Slot.Vault
			else print("höö? '"..r[2].."'")
			end
			for page = tonumber(r[3]), tonumber(r[4]) do
				for slot = tonumber(r[5]), tonumber(r[6]) do
					local pageslot=f(page, slot)
					BankItChar.designations[pageslot]=r[7]
				end
			end
		else
			print (usage)
		end
	else
		print (usage)
	end
end

local function addonLoaded(addon) 
	if (addon == "BankIt") then
		BankIt.printVersion()
		LibVersionCheck.register("BankIt", BankIt.version)
		ensureVariablesInited()
		BankIt.createUI()
	end
end

table.insert(Command.Slash.Register("BankIt"), 		{BankIt.SlashHandler,	"BankIt", "SlashHandler" })
table.insert(Event.Interaction,				{interactionChanged,	"BankIt", "InteractionChanged"})
table.insert(Event.Addon.Load.End, 			{addonLoaded,		"BankIt", "AddonLoaded" })
-- table.insert(Event.System.Update.Begin, 		{systemUpdate,		"BankIt", "systemUpdate"})
table.insert(Event.Item.Slot,		 		{moveComplete,		"BankIt", "moveComplete"})
table.insert(Event.Item.Update,		 		{moveComplete2,		"BankIt", "moveComplete2"})
